@@ -110,36 +110,28 @@ need_fetch_view_set(unsigned i, struct fetch_data_set* fd)
 
 
 /* 6.1.1 CAPABILITY Command */
-/* imap_check_capability: make sure we can log in to this server. */
-static gboolean
-imap_check_capability(ImapMboxHandle* handle)
-{
-  IMAP_REQUIRED_STATE3_U(handle, IMHS_CONNECTED, IMHS_AUTHENTICATED,
-                       IMHS_SELECTED, FALSE);
-  if (imap_cmd_exec(handle, "CAPABILITY") != IMR_OK)
-    return FALSE;
-
-  if (!(imap_mbox_handle_can_do(handle, IMCAP_IMAP4) ||
-        imap_mbox_handle_can_do(handle, IMCAP_IMAP4REV1))) {
-    g_warning("IMAP4rev1 required but not provided.");
-    return FALSE;
-  }  
-  return TRUE;
-}
-
 int
 imap_mbox_handle_can_do(ImapMboxHandle* handle, ImapCapability cap)
 {
+  IMAP_REQUIRED_STATE3_U(handle, IMHS_CONNECTED, IMHS_AUTHENTICATED,
+                         IMHS_SELECTED, IMR_BAD);
   if(cap == IMCAP_FETCHBODY)
     return handle->can_fetch_body;
 
   /* perhaps it already has capabilities? */
-  if(!handle->has_capabilities)
-    imap_check_capability(handle);
+  if(!handle->has_capabilities) {
+    /* If we request CAPABILITY, we have the definitive answer
+     * regardless of the response.
+     */
+    handle->has_capabilities = TRUE;
+    imap_cmd_exec(handle, "CAPABILITY");
+    if (!(handle->capabilities[IMCAP_IMAP4] ||
+          handle->capabilities[IMCAP_IMAP4REV1])) {
+      g_warning("IMAP4rev1 required but not provided.");
+    }
+  }
 
-  if(cap<IMCAP_MAX)
-    return handle->capabilities[cap];
-  else return 0;
+  return (cap<IMCAP_MAX) ? handle->capabilities[cap] : 0;
 }
 
 
@@ -502,8 +494,8 @@ append_commit(ImapMboxHandle *handle, unsigned cmdno, ImapSequence *uid_seq)
       uid_seq->uid_validity = handle->uidplus.dst_uid_validity;
       uid_seq->ranges = handle->uidplus.dst;
     } else if(uid_seq->uid_validity != handle->uidplus.dst_uid_validity) {
-      printf("The IMAP server keeps changing UID validity, "
-	     "ignoring UIDPLUS response (%u -> %u)\n",
+      g_debug("The IMAP server keeps changing UID validity, "
+	     "ignoring UIDPLUS response (%u -> %u)",
 	     uid_seq->uid_validity, handle->uidplus.dst_uid_validity);
       uid_seq->uid_validity = handle->uidplus.dst_uid_validity;
       g_list_free(uid_seq->ranges);
@@ -575,11 +567,11 @@ imap_mbox_append_multi_real(ImapMboxHandle *handle,
       /* MULTIAPPEND continuation */
       if(flags) {
 	gchar *str = enum_flag_to_str(flags);
-	sio_printf(handle->sio, " (%s) {%lu%s}", str,
+	sio_printf(handle->sio, " (%s) {%lu%s}\r\n", str,
 		   (unsigned long)msg_size, litstr);
 	g_free(str);
       } else 
-	sio_printf(handle->sio, " {%lu%s}",
+	sio_printf(handle->sio, " {%lu%s}\r\n",
 		   (unsigned long)msg_size, litstr);
     }
 
@@ -991,7 +983,7 @@ ic_construct_header_list(const char **hdr, ImapFetchType ift)
     if(ift & IMFETCH_LIST_POST)    hdr[idx++] = "LIST-POST";
     hdr[idx++] = ")]";
   }
-  if(ift & IMFETCH_RFC822HEADERS) hdr[idx++] = "RFC822.HEADERS";
+  if(ift & IMFETCH_RFC822HEADERS) hdr[idx++] = "BODY.PEEK[HEADER]";
   if(ift & IMFETCH_RFC822HEADERS_SELECTED)
     hdr[idx++] = "BODY.PEEK[HEADER.FIELDS.NOT (DATE SUBJECT FROM SENDER "
       "REPLY-TO TO CC BCC IN-REPLY-TO MESSAGE-ID)]";
@@ -1177,6 +1169,7 @@ fetch_body_passthrough(unsigned seqno,
   }
 }
 
+/* note: this function is called by imap_tst.c *only* */
 ImapResponse
 imap_mbox_handle_fetch_rfc822(ImapMboxHandle* handle,
 			      unsigned cnt, unsigned *set,
@@ -1198,7 +1191,7 @@ imap_mbox_handle_fetch_rfc822(ImapMboxHandle* handle,
     gchar *cmd = g_strdup_printf("FETCH %s %s", seq,
 				 peek_only
 				 ? "(BODY.PEEK[HEADER] BODY.PEEK[TEXT])"
-				 : "RFC822");
+				 : "BODY[]");
     struct FetchBodyPassthroughData passthrough_data;
     passthrough_data.cb = fetch_cb;
     passthrough_data.arg = fetch_cb_data;
@@ -1333,7 +1326,7 @@ imap_mbox_handle_fetch_rfc822_uid(ImapMboxHandle* handle, unsigned uid,
   } else {
     handle->body_cb  = write_nstring;
     handle->body_arg = fl;
-    cmdstr = "UID FETCH %u RFC822";
+    cmdstr = "UID FETCH %u BODY[]";
   }
 
   snprintf(cmd, sizeof(cmd), cmdstr, uid);
@@ -1858,8 +1851,8 @@ imap_mbox_sort_msgno_client(ImapMboxHandle *handle, ImapSortKey key,
   if(fetch_cnt>0) {
     ImapResponse rc;
     qsort(seqno_to_fetch, fetch_cnt, sizeof(unsigned), comp_unsigned);
-    printf("Should the client side sorting code "
-           "be sorry about your bandwidth usage?\n");
+    g_debug("Should the client side sorting code "
+           "be sorry about your bandwidth usage?");
     rc = imap_mbox_handle_fetch_set_unlocked(handle, seqno_to_fetch,
 					     fetch_cnt, fetch_type);
     if(rc != IMR_OK)
