@@ -36,6 +36,13 @@
 #include "net-client-utils.h"
 #include <glib/gi18n.h>
 
+
+#ifdef G_LOG_DOMAIN
+#  undef G_LOG_DOMAIN
+#endif
+#define G_LOG_DOMAIN "libbalsa-server"
+
+
 #if defined(HAVE_LIBSECRET)
 static const SecretSchema server_schema = {
     "org.gnome.Balsa.NetworkPassword", SECRET_SCHEMA_NONE,
@@ -71,7 +78,7 @@ struct _LibBalsaServerPrivate {
     gchar *cert_passphrase;
     gboolean remember_passwd;
     gboolean remember_cert_passphrase;
-    gboolean try_anonymous; /* user wants anonymous access */
+    NetClientAuthMode auth_mode;
 };
 
 static void libbalsa_server_finalize(GObject * object);
@@ -288,6 +295,7 @@ libbalsa_server_load_security_config(LibBalsaServer *server)
         want_ssl = libbalsa_conf_get_bool_with_default("SSL", &not_found);
         if (want_ssl && !not_found) {
         	priv->security = NET_CLIENT_CRYPT_ENCRYPTED;
+        	libbalsa_conf_clean_key("SSL");
         } else {
         	int want_tls;
 
@@ -305,10 +313,10 @@ libbalsa_server_load_security_config(LibBalsaServer *server)
         		default:
         			priv->security = NET_CLIENT_CRYPT_STARTTLS;
         		}
+            	libbalsa_conf_clean_key("SSL");
         	}
         }
     }
-
 }
 
 
@@ -322,60 +330,74 @@ libbalsa_free_password(gchar *password)
 }
 
 static void
-store_password_libsecret(LibBalsaServer *server,
-	    				 const gchar          *password,
-						 const SecretSchema   *schema)
+store_password_libsecret(LibBalsaServer     *server,
+						 const gchar        *password,
+						 const gchar        *cfg_path,
+						 const SecretSchema *schema)
 {
-    LibBalsaServerPrivate *priv = libbalsa_server_get_instance_private(server);
-    GError *err = NULL;
+	if (libbalsa_conf_use_libsecret()) {
+		LibBalsaServerPrivate *priv = libbalsa_server_get_instance_private(server);
+		GError *err = NULL;
 
-    secret_password_store_sync(schema, NULL, _("Balsa passwords"), password, NULL, &err,
-        "protocol", priv->protocol,
-		"server",   priv->host,
-		"user",     priv->user,
-		NULL);
-    if (err != NULL) {
-        g_info(_("Error storing password for %s@%s:%s: %s"), priv->user, priv->protocol, priv->host, err->message);
-        g_error_free(err);
-    } else {
-        g_debug("stored password for %s@%s:%s in secret service", priv->user, priv->protocol, priv->host);
-    }
+		secret_password_store_sync(schema, NULL, _("Balsa passwords"), password, NULL, &err,
+			"protocol", priv->protocol,
+			"server",   priv->host,
+			"user",     priv->user,
+			NULL);
+		if (err != NULL) {
+			libbalsa_information(LIBBALSA_INFORMATION_ERROR,
+				/* Translators: #1 user name; #2 protocol (imap, etc); #3 server name; #4 error message */
+				_("Error saving credentials for user “%s”, protocol “%s”, server “%s” in Secret Service: %s"),
+				priv->user, priv->protocol, priv->host, err->message);
+			g_error_free(err);
+		} else {
+			g_debug("stored password for %s@%s:%s in secret service", priv->user, priv->protocol, priv->host);
+			libbalsa_conf_clean_key(cfg_path);
+		}
+	} else {
+		libbalsa_conf_private_set_string(cfg_path, password, TRUE);
+	}
 }
 
 static gchar *
-load_password_libsecret(LibBalsaServer *server,
-			  	  	    const gchar          *cfg_path,
-						const SecretSchema   *schema)
+load_password_libsecret(LibBalsaServer     *server,
+						const gchar        *cfg_path,
+						const SecretSchema *schema)
 {
-    LibBalsaServerPrivate *priv = libbalsa_server_get_instance_private(server);
-    GError *err = NULL;
-    gchar *password;
+	gchar *password;
 
-    password = secret_password_lookup_sync(schema, NULL, &err,
-        "protocol", priv->protocol,
-		"server",   priv->host,
-		"user",     priv->user,
-		NULL);
-    if (err != NULL) {
-        g_info(_("Error looking up password for %s@%s:%s: %s"), priv->user, priv->protocol, priv->host, err->message);
-        g_error_free(err);
+	if (libbalsa_conf_use_libsecret()) {
+		LibBalsaServerPrivate *priv = libbalsa_server_get_instance_private(server);
+		GError *err = NULL;
 
-        /* fall back to the private config file */
-        password = libbalsa_conf_private_get_string(cfg_path, TRUE);
-        if ((password != NULL) && (password[0] != '\0')) {
-        g_info(_("loaded fallback password from private config file"));
-        /* store a fallback password in the key ring */
-        store_password_libsecret(server, password, schema);
-            /* We could in principle clear the password in the config file here but we do not for the backward compatibility.
-             * FIXME - Is this really the proper approach, as we leave the obfuscated password in the config file, and the user
-             * relies on the message that the password is stored in the Secret Service only.  OTOH, there are better methods for
-             * protecting the user's secrets, like full disk encryption. */
-        }
-    } else {
-        g_debug("loaded password for %s@%s:%s from secret service", priv->user, priv->protocol, priv->host);
-    }
+		password = secret_password_lookup_sync(schema, NULL, &err,
+			"protocol", priv->protocol,
+			"server",   priv->host,
+			"user",     priv->user,
+			NULL);
+		if (err != NULL) {
+			libbalsa_information(LIBBALSA_INFORMATION_ERROR,
+				/* Translators: #1 user name; #2 protocol (imap, etc); #3 server name; #4 error message */
+				_("Error loading credentials for user %s, protocol “%s”, server “%s” from Secret Service: %s"),
+				priv->user, priv->protocol, priv->host, err->message);
+			g_error_free(err);
+		}
 
-    return password;
+		/* check the config file if the returned password is NULL, make sure to remove it from the config file otherwise */
+		if (password == NULL) {
+			password = libbalsa_conf_private_get_string(cfg_path, TRUE);
+			if (password != NULL) {
+				store_password_libsecret(server, password, cfg_path, schema);
+			}
+		} else {
+			g_debug("loaded password for %s@%s:%s from secret service", priv->user, priv->protocol, priv->host);
+			libbalsa_conf_clean_key(cfg_path);
+		}
+	} else {
+		password = libbalsa_conf_private_get_string(cfg_path, TRUE);
+	}
+
+	return password;
 }
 
 static void
@@ -444,7 +466,13 @@ libbalsa_server_load_config(LibBalsaServer * server)
         priv->user = g_strdup(g_get_user_name());
     }
 
-    priv->try_anonymous = libbalsa_conf_get_bool("Anonymous=false");
+	priv->auth_mode = libbalsa_conf_get_int("AuthMode=2");  /* default NET_CLIENT_AUTH_USER_PASS */
+    if (libbalsa_conf_has_key("Anonymous")) {
+    	if (libbalsa_conf_get_bool("Anonymous")) {
+    		priv->auth_mode = NET_CLIENT_AUTH_NONE_ANON;
+    	}
+    	libbalsa_conf_clean_key("Anonymous");
+    }
     priv->remember_passwd = libbalsa_conf_get_bool("RememberPasswd=false");
 
     priv->passwd = libbalsa_free_password(priv->passwd);
@@ -490,12 +518,12 @@ libbalsa_server_save_config(LibBalsaServer * server)
 
     libbalsa_conf_set_string("Server", priv->host);
     libbalsa_conf_private_set_string("Username", priv->user, FALSE);
-    libbalsa_conf_set_bool("Anonymous",          priv->try_anonymous);
+    libbalsa_conf_set_int("AuthMode", priv->auth_mode);
 
     if (priv->remember_passwd && (priv->passwd != NULL)) {
         libbalsa_conf_set_bool("RememberPasswd", TRUE);
 #if defined(HAVE_LIBSECRET)
-        store_password_libsecret(server, priv->passwd, &server_schema);
+        store_password_libsecret(server, priv->passwd, "Password", &server_schema);
 #else
         libbalsa_conf_private_set_string("Password", priv->passwd, TRUE);
 #endif
@@ -513,7 +541,7 @@ libbalsa_server_save_config(LibBalsaServer * server)
         if (priv->remember_cert_passphrase && (priv->cert_passphrase != NULL)) {
         	libbalsa_conf_set_bool("RememberCertPasswd", TRUE);
 #if defined(HAVE_LIBSECRET)
-        	store_password_libsecret(server, priv->cert_passphrase, &cert_pass_schema);
+        	store_password_libsecret(server, priv->cert_passphrase, "CertificatePassphrase", &cert_pass_schema);
 #else
         	libbalsa_conf_private_set_string("CertificatePassphrase", priv->cert_passphrase, TRUE);
 #endif
@@ -546,25 +574,30 @@ libbalsa_server_connect_signals(LibBalsaServer * server, GCallback cb,
 
 
 gchar **
-libbalsa_server_get_auth(NetClient *client,
-						 gboolean   need_passwd,
-         	 	 	 gpointer   user_data)
+libbalsa_server_get_auth(NetClient         *client,
+						 NetClientAuthMode  mode,
+						 gpointer           user_data)
 {
     LibBalsaServer *server = LIBBALSA_SERVER(user_data);
     LibBalsaServerPrivate *priv = libbalsa_server_get_instance_private(server);
     gchar **result = NULL;
 
-    g_debug("%s: %p %p: encrypted = %d", __func__, client, user_data,
-            net_client_is_encrypted(client));
-    if (!priv->try_anonymous || (strcmp(priv->protocol, "imap") == 0)) {
+    g_debug("%s: %p %d %p: encrypted = %d", __func__, client, mode, user_data, net_client_is_encrypted(client));
+    if (priv->auth_mode != NET_CLIENT_AUTH_NONE_ANON) {
         result = g_new0(gchar *, 3U);
         result[0] = g_strdup(priv->user);
-        if (need_passwd) {
-        if ((priv->passwd != NULL) && (priv->passwd[0] != '\0')) {
-        	result[1] = g_strdup(priv->passwd);
-        } else {
-        	result[1] = lbs_get_password(server, NULL);
-        }
+        switch (mode) {
+        case NET_CLIENT_AUTH_USER_PASS:
+            if ((priv->passwd != NULL) && (priv->passwd[0] != '\0')) {
+            	result[1] = g_strdup(priv->passwd);
+            } else {
+            	result[1] = lbs_get_password(server, NULL);
+            }
+            break;
+        case NET_CLIENT_AUTH_KERBEROS:
+        	break;			/* only user name required */
+        default:
+        	g_assert_not_reached();
         }
     }
     return result;
@@ -630,7 +663,6 @@ libbalsa_server_test_can_reach_full(LibBalsaServer           * server,
     LibBalsaServerPrivate *priv = libbalsa_server_get_instance_private(server);
     CanReachInfo *info;
     gchar *host;
-    gchar *colon;
     GNetworkMonitor *monitor;
     GSocketConnectable *address;
 
@@ -641,11 +673,7 @@ libbalsa_server_test_can_reach_full(LibBalsaServer           * server,
 
     monitor = g_network_monitor_get_default();
 
-    host = g_strdup(priv->host);
-    colon = strchr(host, ':');
-    if (colon != NULL) {
-        colon[0] = '\0';
-    }
+    host = net_client_host_only(priv->host);
     address = g_network_address_new(host, 0);
     g_free(host);
     g_network_monitor_can_reach_async(monitor, address, NULL,
@@ -735,14 +763,14 @@ libbalsa_server_get_security(LibBalsaServer *server)
     return priv->security;
 }
 
-gboolean
-libbalsa_server_get_try_anonymous(LibBalsaServer *server)
+NetClientAuthMode
+libbalsa_server_get_auth_mode(LibBalsaServer *server)
 {
     LibBalsaServerPrivate *priv = libbalsa_server_get_instance_private(server);
 
-    g_return_val_if_fail(LIBBALSA_IS_SERVER(server), FALSE);
+    g_return_val_if_fail(LIBBALSA_IS_SERVER(server), (NetClientAuthMode) 0);
 
-    return priv->try_anonymous;
+    return priv->auth_mode;
 }
 
 gboolean
@@ -813,13 +841,13 @@ libbalsa_server_set_security(LibBalsaServer *server, NetClientCryptMode security
 }
 
 void
-libbalsa_server_set_try_anonymous(LibBalsaServer *server, gboolean try_anonymous)
+libbalsa_server_set_auth_mode(LibBalsaServer *server, NetClientAuthMode auth_mode)
 {
     LibBalsaServerPrivate *priv = libbalsa_server_get_instance_private(server);
 
     g_return_if_fail(LIBBALSA_IS_SERVER(server));
 
-    priv->try_anonymous = try_anonymous;
+    priv->auth_mode = auth_mode;
 }
 
 void
