@@ -34,10 +34,6 @@
 #include "smtp-server.h"
 #include "save-restore.h"
 
-#if HAVE_MACOSX_DESKTOP
-#  include "macosx-helpers.h"
-#endif
-
 #if HAVE_GCR
 #define GCR_API_SUBJECT_TO_CHANGE
 #include <gcr/gcr.h>
@@ -55,13 +51,10 @@ struct BalsaApplication balsa_app;
 static gchar *
 ask_password_real(LibBalsaServer *server, const gchar *cert_subject)
 {
-#if defined(HAVE_LIBSECRET)
-    static const gchar *remember_password_message =
-        N_("_Remember password in Secret Service");
-#else
-    static const gchar *remember_password_message =
-        N_("_Remember password");
-#endif                          /* defined(HAVE_LIBSECRET) */
+    static const gchar *remember_password_message[2] = {
+        N_("_Remember password in Secret Service"),
+        N_("_Remember password")
+    };
 	GcrPromptDialog *dialog;
 	gchar *prompt;
 	gboolean remember;
@@ -69,28 +62,32 @@ ask_password_real(LibBalsaServer *server, const gchar *cert_subject)
         const gchar *user;
         const gchar *host;
         const gchar *protocol;
+        int use_remember_idx;
 
     g_return_val_if_fail(server != NULL, NULL);
 
+#if defined(HAVE_LIBSECRET)
+    use_remember_idx = libbalsa_conf_use_libsecret() ? 0 : 1;
+#else
+    use_remember_idx = 1;
+#endif                          /* defined(HAVE_LIBSECRET) */
     user = libbalsa_server_get_user(server);
     host = libbalsa_server_get_host(server);
     protocol = libbalsa_server_get_protocol(server);
     if (cert_subject != NULL) {
-    	prompt = g_strdup_printf(_("Password to unlock the user certificate\n%s\nfor %s@%s (%s)"),
-    		cert_subject,
-                user, host, protocol);
+		/* Translators: #1 Certificate CN, #2 user name; #3 protocol (imap, etc); #4 server name */
+    	prompt = g_strdup_printf(_("Password to unlock the user certificate\n%s\nfor user %s, protocol “%s”, server “%s”"),
+    		cert_subject, user, protocol, host);
     	remember = libbalsa_server_get_remember_cert_passphrase(server);
     } else {
-    	prompt = g_strdup_printf(_("Password for %s@%s (%s)"),
-                user, host, protocol);
+		/* Translators: #1 user name; #2 protocol (imap, etc); #3 server name */
+    	prompt = g_strdup_printf(_("Password for user %s, protocol “%s”, server “%s”"),
+                user, protocol, host);
     	remember = libbalsa_server_get_remember_password(server);
     }
 	dialog = g_object_new(GCR_TYPE_PROMPT_DIALOG,
 						  "use-header-bar", libbalsa_use_headerbar(),
 						  "title", _("Password needed"),
-						  "description", prompt,
-						  "message", _("Password needed"),
-						  "choice-label", remember_password_message,
 						  "cancel-label", _("_Cancel"),
 						  "continue-label", _("_OK"),
 						  "choice-chosen", remember,
@@ -98,6 +95,11 @@ ask_password_real(LibBalsaServer *server, const gchar *cert_subject)
 						  "transient-for", GTK_WINDOW(balsa_app.main_window),
 						  "modal", TRUE,
 						  NULL);
+	/* for some strange reason, the following property strings are not displayed when they are
+	 * added to the call above - no idea why */
+	gcr_prompt_set_message(GCR_PROMPT(dialog), _("Password needed"));
+	gcr_prompt_set_description(GCR_PROMPT(dialog), prompt);
+	gcr_prompt_set_choice_label(GCR_PROMPT(dialog),remember_password_message[use_remember_idx]);
 	g_free(prompt);
 	passwd = g_strdup(gcr_prompt_password_run(GCR_PROMPT(dialog), NULL, NULL));
 	if (passwd != NULL) {
@@ -120,8 +122,6 @@ ask_password_real(LibBalsaServer *server, const gchar *cert_subject)
 
 #else
 
-#define HIG_PADDING 12
-
 /* ask_password:
    asks the user for the password to the mailbox on given remote server.
 */
@@ -137,21 +137,22 @@ ask_password_real(LibBalsaServer * server, const gchar *cert_subject)
     gchar *prompt;
     gchar *passwd;
 	gboolean remember;
-#if defined(HAVE_LIBSECRET)
-    static const gchar *remember_password_message =
-        N_("_Remember password in Secret Service");
-#else
-    static const gchar *remember_password_message =
-        N_("_Remember password");
-#endif                          /* defined(HAVE_LIBSECRET) */
+    int use_remember_idx;
+    static const gchar *remember_password_message[2] = {
+        N_("_Remember password in Secret Service"),
+        N_("_Remember password")
+    };
 
     g_return_val_if_fail(server != NULL, NULL);
     if (cert_subject != NULL) {
-    	prompt = g_strdup_printf(_("Password to unlock the user certificate\n%s\nfor %s@%s (%s)"),
-    		cert_subject, libbalsa_server_get_user(server), libbalsa_server_get_host(server), libbalsa_server_get_protocol(server));
+		/* Translators: #1 Certificate CN, #2 user name; #3 protocol (imap, etc); #4 server name */
+    	prompt = g_strdup_printf(_("Password to unlock the user certificate\n%s\nfor user %s, protocol “%s”, server “%s”"),
+    		cert_subject, libbalsa_server_get_user(server), libbalsa_server_get_protocol(server), libbalsa_server_get_host(server));
     	remember = libbalsa_server_get_remember_cert_passphrase(server);
     } else {
-    	prompt = g_strdup_printf(_("Password for %s@%s (%s)"), libbalsa_server_get_user(server), libbalsa_server_get_host(server), libbalsa_server_get_protocol(server));
+		/* Translators: #1 user name; #2 protocol (imap, etc); #3 server name */
+    	prompt = g_strdup_printf(_("Password for user %s, protocol “%s”, server “%s”"),
+    		libbalsa_server_get_user(server), libbalsa_server_get_protocol(server), libbalsa_server_get_host(server));
        	remember = libbalsa_server_get_remember_password(server);
     }
 
@@ -162,12 +163,9 @@ ask_password_real(LibBalsaServer * server, const gchar *cert_subject)
                                          _("_OK"), GTK_RESPONSE_OK,
                                          _("_Cancel"), GTK_RESPONSE_CANCEL,
                                          NULL); 
-#if HAVE_MACOSX_DESKTOP
-    libbalsa_macosx_menu_for_parent(dialog, GTK_WINDOW(balsa_app.main_window));
-#endif
 
     content = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
-    gtk_container_set_border_width(GTK_CONTAINER(content), HIG_PADDING);
+    gtk_container_set_border_width(GTK_CONTAINER(content), 2 * HIG_PADDING);
 
     grid = libbalsa_create_grid();
     gtk_container_add(GTK_CONTAINER(content), grid);
@@ -177,12 +175,16 @@ ask_password_real(LibBalsaServer * server, const gchar *cert_subject)
 
     label = libbalsa_create_grid_label(_("Password:"), grid, 1);
     entry = libbalsa_create_grid_entry(grid, NULL, NULL, 1, NULL, label);
-    g_object_set(entry, "input-purpose", GTK_INPUT_PURPOSE_PASSWORD, NULL);
-    gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
+    libbalsa_entry_config_passwd(GTK_ENTRY(entry));
     gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);
     gtk_widget_grab_focus(entry);
 
-    rememb_check = libbalsa_create_grid_check(remember_password_message, grid, 2, remember);
+#if defined(HAVE_LIBSECRET)
+    use_remember_idx = libbalsa_conf_use_libsecret() ? 0 : 1;
+#else
+    use_remember_idx = 1;
+#endif                          /* defined(HAVE_LIBSECRET) */
+    rememb_check = libbalsa_create_grid_check(remember_password_message[use_remember_idx], grid, 2, remember);
 
     gtk_widget_show_all(grid);
     gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
@@ -318,7 +320,6 @@ balsa_app_init(void)
     balsa_app.check_mail_auto = TRUE;
     balsa_app.check_mail_timer = 10;
 
-    balsa_app.debug = FALSE;
     balsa_app.previewpane = TRUE;
     balsa_app.pgdownmod = FALSE;
     balsa_app.pgdown_percent = 50;
@@ -413,9 +414,17 @@ balsa_app_init(void)
     balsa_app.error_message = 0;
     balsa_app.debug_message = 0;
 
-    balsa_app.notify_new_mail_sound = 1;
+#ifdef HAVE_CANBERRA
+    balsa_app.notify_new_mail_sound = 0;
+#endif
+
     balsa_app.notify_new_mail_dialog = 0;
-    balsa_app.notify_new_mail_icon = 1;
+
+#ifdef ENABLE_SYSTRAY
+    balsa_app.enable_systray_icon = 0;
+#endif
+
+    balsa_app.enable_dkim_checks = 0;
 
     /* Local and IMAP */
     balsa_app.local_scan_depth = 1;
@@ -462,7 +471,7 @@ balsa_app_destroy(void)
     balsa_app.fcc_mru = NULL;
 
 
-    if(balsa_app.debug) g_print("balsa_app: Finished cleaning up.\n");
+    g_debug("balsa_app: Finished cleaning up.");
 }
 
 static gboolean
@@ -470,8 +479,7 @@ check_new_messages_auto_cb(gpointer data)
 {
     check_new_messages_real(balsa_app.main_window, TRUE);
 
-    if (balsa_app.debug)
-        fprintf(stderr, "Auto-checked for new messages…\n");
+    g_debug("Auto-checked for new messages…");
 
     /*  preserver timer */
     return TRUE;
@@ -520,9 +528,7 @@ open_mailbox_by_url(const gchar * url, gboolean hidden)
     LibBalsaMailbox *mailbox;
 
     mailbox = balsa_find_mailbox_by_url(url);
-    if (balsa_app.debug)
-        fprintf(stderr, "balsa_open_mailbox_list: opening %s => %p..\n",
-                url, mailbox);
+    g_debug("balsa_open_mailbox_list: opening %s => %p.", url, mailbox);
     if (mailbox) {
         if (hidden)
             balsa_mblist_open_mailbox_hidden(mailbox);
@@ -852,7 +858,7 @@ ba_remove_children_mailbox_nodes(GtkTreeModel * model, GtkTreeIter * parent,
 	    valid =
 		gtk_tree_store_remove(balsa_app.mblist_tree_store, &iter);
 	} else {
-	    printf("sparing %s %s\n",
+	    g_debug("sparing %s %s",
 		   mailbox != NULL ? "mailbox" : "folder ",
 		   mailbox != NULL ?
                    libbalsa_mailbox_get_name(mailbox) :
@@ -870,14 +876,11 @@ balsa_remove_children_mailbox_nodes(BalsaMailboxNode * mbnode)
     GtkTreeIter parent;
     GtkTreeIter *iter = NULL;
     GSList *specials = NULL, *l;
+    const gchar *name;
 
-    if (balsa_app.debug) {
-        const gchar *name;
-
-	printf("Destroying children of %p %s\n", mbnode,
+	g_debug("Destroying children of %p %s", mbnode,
                (mbnode != NULL &&
                 (name = balsa_mailbox_node_get_name(mbnode)) != NULL) ? name : "");
-    }
 
     if (mbnode && balsa_find_iter_by_data(&parent, mbnode))
 	iter = &parent;
@@ -947,3 +950,20 @@ balsa_quote_regex_new(void)
 
     return g_regex_ref(regex);
 }
+
+gboolean
+balsa_autocrypt_in_use(void)
+{
+	gboolean result = FALSE;
+#ifdef ENABLE_AUTOCRYPT
+	GList *ident;
+
+	for (ident = balsa_app.identities; !result && (ident != NULL); ident = ident->next) {
+                LibBalsaIdentity *identity = LIBBALSA_IDENTITY(ident->data);
+                AutocryptMode autocrypt_mode = libbalsa_identity_get_autocrypt_mode(identity);
+		result = autocrypt_mode != AUTOCRYPT_DISABLE;
+	}
+#endif  /* ENABLE_AUTOCRYPT */
+	return result;
+}
+
