@@ -93,6 +93,7 @@ balsa_print_object_header_new_real(GList * list,
 				   GtkPrintContext * context,
 				   LibBalsaMessageBody * sig_body,
 				   LibBalsaMessageHeaders * headers,
+				   LibBalsaDkim *dkim,
 				   const gchar * the_subject,
 				   BalsaPrintSetup * psetup,
 				   gboolean print_all_headers)
@@ -149,6 +150,8 @@ balsa_print_object_header_new_real(GList * list,
 		    headers->cc_list, &p_label_width, print_all_headers);
     header_add_list(test_layout, header_buf, "bcc", _("BCC:"),
 		    headers->bcc_list, &p_label_width, print_all_headers);
+    header_add_list(test_layout, header_buf, "sender", _("Sender:"),
+		    headers->sender, &p_label_width, print_all_headers);
     header_add_string(test_layout, header_buf, "fcc", _("FCC:"),
 		      headers->fcc_url, &p_label_width, print_all_headers);
     header_add_list(test_layout, header_buf, "disposition-notification-to",
@@ -210,6 +213,10 @@ balsa_print_object_header_new_real(GList * list,
     	    g_string_append_printf(header_buf, "%s\n", info_str);
     	    g_free(info_str);
     	}
+
+        if ((balsa_app.enable_dkim_checks != 0) && (libbalsa_dkim_status_code(dkim) >= DKIM_SUCCESS)) {
+        	g_string_append_printf(header_buf, "%s\n", libbalsa_dkim_status_str_short(dkim));
+        }
     }
 
     /* strip the trailing '\n' */
@@ -300,6 +307,7 @@ balsa_print_object_header_from_message(GList *list,
     return balsa_print_object_header_new_real(list, context,
 					      libbalsa_message_get_body_list(message),
 					      libbalsa_message_get_headers(message),
+						  libbalsa_message_get_body_list(message)->dkim,
                                               subject, psetup, FALSE);
 }
 
@@ -311,7 +319,7 @@ balsa_print_object_header_from_body(GList *list,
 				    BalsaPrintSetup * psetup)
 {
     return balsa_print_object_header_new_real(list, context, body->parts,
-					      body->embhdrs,
+					      body->embhdrs, body->dkim,
 					      body->embhdrs->subject,
 					      psetup,
 					      body->body_type == LIBBALSA_MESSAGE_BODY_TYPE_TEXT);
@@ -492,7 +500,7 @@ header_add_list(PangoLayout * layout, GString * header_buf,
 	  balsa_app.shown_headers == HEADERS_ALL ||
 	  libbalsa_find_word(field_id, balsa_app.selected_headers)) ||
         !values ||
-        !(_value = internet_address_list_to_string(values, FALSE)))
+        !(_value = internet_address_list_to_string(values, NULL, FALSE)))
 	return;
 
     p_width = p_string_width_from_layout(layout, label);
