@@ -39,10 +39,6 @@
 #include "libbalsa-conf.h"
 #include "missing.h"
 
-#if HAVE_MACOSX_DESKTOP
-#  include "macosx-helpers.h"
-#endif
-
 /* Defined in filter-edit-dialog.c*/
 extern option_list fe_search_type[4];
 extern GList * fe_user_headers_list;
@@ -835,9 +831,6 @@ condition_dialog_response(GtkWidget * dialog, gint response,
 {
     LibBalsaCondition *new_cnd;
     GError *err = NULL;
-#if !GTK_CHECK_VERSION(3, 22, 0)
-    GdkScreen *screen;
-#endif /* GTK_CHECK_VERSION(3, 22, 0) */
 
     switch (response) {
     case GTK_RESPONSE_OK:       /* OK button */
@@ -893,15 +886,9 @@ condition_dialog_response(GtkWidget * dialog, gint response,
         gtk_widget_hide(dialog);
         break;
     case GTK_RESPONSE_HELP:     /* Help button */
-#if GTK_CHECK_VERSION(3, 22, 0)
         gtk_show_uri_on_window(GTK_WINDOW(dialog),
                                "help:balsa/win-filters#win-condition",
                                gtk_get_current_event_time(), &err);
-#else  /* GTK_CHECK_VERSION(3, 22, 0) */
-        screen = gtk_widget_get_screen(dialog);
-        gtk_show_uri(screen, "help:balsa/win-filters#win-condition",
-                     gtk_get_current_event_time(), &err);
-#endif  /* GTK_CHECK_VERSION(3, 22, 0) */
 	if (err) {
 	    balsa_information_parented(GTK_WINDOW(dialog),
 		    LIBBALSA_INFORMATION_WARNING,
@@ -1008,12 +995,14 @@ build_type_notebook()
     /* The simple page of the notebook */
 
     grid = gtk_grid_new();
-    gtk_grid_set_row_spacing(GTK_GRID(grid), 5);
-    gtk_grid_set_column_spacing(GTK_GRID(grid), 5);
+    gtk_grid_set_row_spacing(GTK_GRID(grid), HIG_PADDING);
+    gtk_grid_set_column_spacing(GTK_GRID(grid), HIG_PADDING);
 
-    box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
-    g_object_set(box, "margin", 5, NULL);
-    gtk_box_pack_start(GTK_BOX(box), grid, TRUE, TRUE, 0);
+    box = gtk_box_new(GTK_ORIENTATION_VERTICAL, HIG_PADDING);
+    libbalsa_set_margins(box, HIG_PADDING);
+    gtk_widget_set_vexpand(grid, TRUE);
+    gtk_widget_set_valign(grid, GTK_ALIGN_FILL);
+    gtk_container_add(GTK_CONTAINER(box), grid);
 
     fe_type_simple_label = 
 	gtk_label_new_with_mnemonic(_("One of the specified f_ields contains"));
@@ -1036,14 +1025,16 @@ build_type_notebook()
     gtk_notebook_append_page(GTK_NOTEBOOK(fe_type_notebook), box, NULL);
     
     /* The regex page of the type notebook */
-    box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
-    g_object_set(box, "margin", 5, NULL);
+    box = gtk_box_new(GTK_ORIENTATION_VERTICAL, HIG_PADDING);
+    libbalsa_set_margins(box, HIG_PADDING);
 
     gtk_notebook_append_page(GTK_NOTEBOOK(fe_type_notebook), box, NULL);
 
 #if REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED
     page = gtk_table_new(5, 6, FALSE);
-    gtk_box_pack_start(GTK_BOX(box), page, TRUE, TRUE, 0);
+    gtk_widget_set_vexpand(page, TRUE);
+    gtk_widget_set_valign(page, GTK_ALIGN_FILL);
+    gtk_container_add(GTK_CONTAINER(box), page);
 
     fe_type_regex_label = 
         gtk_label_new_with_mnemonic(_("_One of the regular expressions matches"));
@@ -1074,16 +1065,25 @@ build_type_notebook()
                      box,
                      0, 5, 4, 5,
                      GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 2, 2);
+
     button = gtk_button_new_with_mnemonic(_("A_dd"));
-    gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
+    gtk_widget_set_hexpand(button, TRUE);
+    gtk_widget_set_halign(button, GTK_ALIGN_FILL);
+    gtk_container_add(GTK_CONTAINER(box), button);
     g_signal_connect(button, "clicked",
                      G_CALLBACK(fe_add_pressed), NULL);
+
     fe_regex_remove_button = gtk_button_new_with_mnemonic(_("_Remove"));
-    gtk_box_pack_start(GTK_BOX(box), fe_regex_remove_button, TRUE, TRUE, 0);
+    gtk_widget_set_hexpand(fe_regex_remove_button, TRUE);
+    gtk_widget_set_halign(fe_regex_remove_button, GTK_ALIGN_FILL);
+    gtk_container_add(GTK_CONTAINER(box), fe_regex_remove_button);
     g_signal_connect(fe_regex_remove_button, "clicked",
                      G_CALLBACK(fe_remove_pressed), NULL);
+
     button = gtk_button_new_with_mnemonic(_("One _Matches/None Matches"));
-    gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
+    gtk_widget_set_hexpand(button, TRUE);
+    gtk_widget_set_halign(button, GTK_ALIGN_FILL);
+    gtk_container_add(GTK_CONTAINER(box), button);
     g_signal_connect(button, "clicked",
                      G_CALLBACK(fe_negate_condition), NULL);
 
@@ -1095,15 +1095,17 @@ build_type_notebook()
 #else                   /* REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED */
     msg = _("Filtering using regular expressions is not yet implemented.");
     label = gtk_label_new(msg);
-    gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 0);
+    gtk_widget_set_hexpand(label, TRUE);
+    gtk_widget_set_halign(label, GTK_ALIGN_FILL);
+    gtk_container_add(GTK_CONTAINER(box), label);
 #endif                  /* REGULAR_EXPRESSION_FILTERING_IS_IMPLEMENTED */
 
     /* The date page of the notebook */
 
     page = gtk_grid_new();
-    gtk_grid_set_row_spacing(GTK_GRID(page), 5);
-    gtk_grid_set_column_spacing(GTK_GRID(page), 5);
-    g_object_set(page, "margin", 5, NULL);
+    gtk_grid_set_row_spacing(GTK_GRID(page), HIG_PADDING);
+    gtk_grid_set_column_spacing(GTK_GRID(page), HIG_PADDING);
+    libbalsa_set_margins(page, HIG_PADDING);
 
     gtk_notebook_append_page(GTK_NOTEBOOK(fe_type_notebook), page, NULL);
 
@@ -1133,9 +1135,9 @@ build_type_notebook()
 
     page = gtk_grid_new();
     gtk_widget_set_vexpand(page, TRUE);
-    gtk_grid_set_row_spacing(GTK_GRID(page), 5);
-    gtk_grid_set_column_spacing(GTK_GRID(page), 5);
-    g_object_set(page, "margin", 5, NULL);
+    gtk_grid_set_row_spacing(GTK_GRID(page), HIG_PADDING);
+    gtk_grid_set_column_spacing(GTK_GRID(page), HIG_PADDING);
+    libbalsa_set_margins(page, HIG_PADDING);
 
     gtk_notebook_append_page(GTK_NOTEBOOK(fe_type_notebook), page, NULL);
     fe_type_flag_label =
@@ -1176,26 +1178,34 @@ void build_condition_dialog(GtkWidget * condition_dialog)
 {
     GtkWidget *label,* box;
     GtkWidget *field_frame = get_field_frame();
-    GtkBox *content_box;
+    GtkWidget *content_box;
 
     /* builds the toggle buttons to specify fields concerned by the
      * conditions of the filter */
-    box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5);
+    box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, HIG_PADDING);
+
     label = gtk_label_new_with_mnemonic(_("Search T_ype:"));
-    gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 5);
-    fe_search_option_menu = 
+    libbalsa_set_hmargins(label, HIG_PADDING);
+    gtk_container_add(GTK_CONTAINER(box), label);
+
+    fe_search_option_menu =
         fe_build_option_menu(fe_search_type, G_N_ELEMENTS(fe_search_type),
                              G_CALLBACK(fe_typesmenu_cb), field_frame);
-    gtk_box_pack_start(GTK_BOX(box), fe_search_option_menu, FALSE, FALSE, 5);
+    libbalsa_set_hmargins(fe_search_option_menu, HIG_PADDING);
+    gtk_container_add(GTK_CONTAINER(box), fe_search_option_menu);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), fe_search_option_menu);
 
-    content_box =
-        GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(condition_dialog)));
-    gtk_box_pack_start(content_box, box, FALSE, FALSE, 2);
-    gtk_box_pack_start(content_box, field_frame, FALSE, FALSE, 2);
+    content_box = gtk_dialog_get_content_area(GTK_DIALOG(condition_dialog));
+
+    libbalsa_set_vmargins(box, 2);
+    gtk_container_add(GTK_CONTAINER(content_box), box);
+
+    libbalsa_set_vmargins(field_frame, 2);
+    gtk_container_add(GTK_CONTAINER(content_box), field_frame);
 
     build_type_notebook();
-    gtk_box_pack_start(content_box, fe_type_notebook, FALSE, FALSE, 2);
+    libbalsa_set_vmargins(fe_type_notebook, 2);
+    gtk_container_add(GTK_CONTAINER(content_box), fe_type_notebook);
 }
 
 /*
@@ -1247,9 +1257,6 @@ fe_edit_condition(GtkWidget * throwaway,gpointer is_new_cnd)
                                         _("_Cancel"), GTK_RESPONSE_CANCEL,
                                         _("_Help"), GTK_RESPONSE_HELP,
                                         NULL);
-#if HAVE_MACOSX_DESKTOP
-	libbalsa_macosx_menu_for_parent(condition_dialog, GTK_WINDOW(fe_window));
-#endif
         g_object_add_weak_pointer(G_OBJECT(condition_dialog),
                                   (gpointer *) &condition_dialog);
 
@@ -1464,9 +1471,6 @@ fe_dialog_response(GtkWidget * dialog, gint response, gpointer data)
     GtkTreeIter iter;
     gboolean valid;
     GError *err = NULL;
-#if !GTK_CHECK_VERSION(3, 22, 0)
-    GdkScreen *screen;
-#endif /* GTK_CHECK_VERSION(3, 22, 0) */
 
     switch (response) {
     case GTK_RESPONSE_OK:       /* OK button */
@@ -1505,14 +1509,8 @@ fe_dialog_response(GtkWidget * dialog, gint response, gpointer data)
         break;
 
     case GTK_RESPONSE_HELP:     /* Help button */
-#if GTK_CHECK_VERSION(3, 22, 0)
         gtk_show_uri_on_window(GTK_WINDOW(dialog), "help:balsa/win-filters",
                                    gtk_get_current_event_time(), &err);
-#else  /* GTK_CHECK_VERSION(3, 22, 0) */
-        screen = gtk_widget_get_screen(dialog);
-        gtk_show_uri(screen, "help:balsa/win-filters",
-                     gtk_get_current_event_time(), &err);
-#endif  /* GTK_CHECK_VERSION(3, 22, 0) */
 	if (err) {
 	    balsa_information_parented(GTK_WINDOW(dialog),
 		    LIBBALSA_INFORMATION_WARNING,
@@ -2162,7 +2160,7 @@ fe_date_sample(void)
 
 /* Callback for the sound file-chooser-button's dialog. */
 void
-fe_sound_response(GtkDialog * dialog, gint response)
+fe_sound_response(GtkDialog * dialog, gint response, gpointer data)
 {
     if (response == GTK_RESPONSE_ACCEPT)
         set_button_sensitivities(TRUE);
